﻿import datetime
from encodings import utf_8
import es
import gamethread
import langlib
import math
import os
import playerlib
import popuplib
import random
import sqlite3
import time

database_path = "cstrike/cfg/soccer_mod/ranking.db"
lang_dir = "cstrike/addons/eventscripts/soccer_mod_2010/language/"

common = langlib.Strings(lang_dir + "popup_common.ini")
messages = langlib.Strings(lang_dir + "match/messages_match_stats.ini")
messages_common = langlib.Strings(lang_dir + "messages_common.ini")
text = langlib.Strings(lang_dir + "match/popup_stats.ini")

database_exists = os.path.isfile(database_path)

connection = sqlite3.connect(database_path)
cursor = connection.cursor()

if database_exists != True:
	create_table = "CREATE TABLE [players] ("
	create_table += "[steamid] VARCHAR(255)  PRIMARY KEY NOT NULL,"
	create_table += "[name] VARCHAR(255)  NOT NULL,"
	create_table += "[connected] VARCHAR(255)  NOT NULL,"
	create_table += "[goals] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[assists] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[own_goals] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[hits] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[passes] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[interceptions] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[losses] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[rank] INTEGER DEFAULT '0' NOT NULL,"
	create_table += "[time] INTEGER DEFAULT '0' NOT NULL"
	create_table += ")"
	cursor.execute(create_table)

exists = 0
cursor.execute("PRAGMA table_info(players)")
for row in cursor:
	for item in row:
		if item == "time":
			exists = 1

if exists == 0:
	cursor.execute("ALTER TABLE players ADD time INTEGER DEFAULT '0' NOT NULL")

# ******************** Load ********************
def load():
	load_variables()
	reset_team_stats()

	es.keygroupcreate("soccer_mod_stats")

	create_popup()

	cmd = "sm_event_stats_%s" % (random.randint(1, 99999))
	es.set("sm_event_stats", cmd)
	es.regclientcmd(cmd, "soccer_mod_2010/match/stats/ball_hit")

	es.load("soccer_mod_2010/match/stats/ranking")
	es.load("soccer_mod_2010/match/stats/score_sheet")

# ******************** Events ********************
def es_map_start(event_var):
	connection.commit()

	load_variables()
	reset_team_stats()

	es.keygroupdelete("soccer_mod_stats")
	es.keygroupcreate("soccer_mod_stats")

def player_activate(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		player = playerlib.getPlayer(event_var["userid"])
		if player.steamid != "BOT":
			set_connected(player.name, player.steamid)

def player_disconnect(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if len(playerlib.getPlayerList("#human")) < 1:
			connection.commit()

		#player = playerlib.getPlayer(event_var["userid"])
		#if player.steamid != "BOT":
			#set_connected(player.name, player.steamid)

def player_spawn(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		if int(event_var["es_userteam"]) > 1:
			if es.exists("key", "soccer_mod_stats", event_var["es_steamid"]) == 1:
				es.server.queuecmd("score set %s %s" % (event_var["userid"], es.keygetvalue("soccer_mod_stats", event_var["es_steamid"], "goals")))
			else:
				es.server.queuecmd("score set %s 0" % (event_var["userid"]))

def round_start(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		es.set("sm_match_first_hit", 1)
		es.set("sm_stats_round_end", 0)
		es.set("sm_match_goal_scored", 0)

		load_variables()

		players = playerlib.getUseridList("#all")
		if len(players) > 0:
			userid = random.choice(players)

			es.give(userid, "point_clientcommand")
			event = es.ServerVar("sm_event_stats")

			es.server.queuecmd('es_xfire %s func_physbox addoutput "OnDamaged point_clientcommand,command,%s"' % (userid, event))
			es.server.queuecmd('es_xfire %s prop_physics addoutput "OnTakeDamage point_clientcommand,command,%s"' % (userid, event))
			es.server.queuecmd('es_xfire %s trigger_hurt addoutput "OnStartTouch point_servercommand,command,es_xset sm_match_goal_scored 1"' % (userid))

def round_end(event_var):
	if es.ServerVar("soccer_mod_allowed") == 1:
		gamethread.delayed(0.15, show_stats, (int(event_var["winner"])))

# ******************** Load variables ********************
def load_variables():
	es.set("sm_stats_shooter_userid", 0)
	es.set("sm_stats_shooter_name", 0)
	es.set("sm_stats_shooter_team", 0)
	es.set("sm_stats_shooter_steamid", 0)

	global shooter_time_stamp
	shooter_time_stamp = 0

	es.set("sm_stats_assister_userid", 0)
	es.set("sm_stats_assister_name", 0)
	es.set("sm_stats_assister_team", 0)
	es.set("sm_stats_assister_steamid", 0)

	global assister_time_stamp
	assister_time_stamp = 0

	es.set("sm_stats_winner", 0)

# ******************** Create popup ********************
def create_popup():
	popup = popuplib.create("sm_popup_stats")

	popup.addline("SOCCER MOD %s" % (text("title")))
	popup.addline(" ")

	values = ["team", "personal", "player", "ranking", "score_sheet"]
	for i in range(0, 5):
		popup.addline("->%s. %s" % (i + 1, text(values[i])))

	popup.addline(" ")
	popup.addline("0. %s" % (common("close")))

	popup.select(1, team)
	popup.select(2, personal)
	popup.select(3, player)
	popup.submenu(4, "sm_popup_stats_ranking")
	popup.select(5, score_sheet)

	for i in range(6, 10):
		popup.submenu(i, "sm_popup_stats")

# ******************** Commands ********************
def cmd_stats():
	tokens = {}
	userid = es.getcmduserid()
	player = playerlib.getPlayer(userid)

	if es.ServerVar("soccer_mod_allowed") == 1:
		popuplib.send("sm_popup_stats", userid)
	else:
		es.tell(userid, "#green", messages_common("not_allowed", tokens, player.get("lang")))

# ******************** Popup choice ********************
def team(userid, choice, popupid):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	popup = popuplib.create(name)

	popup.addline("SOCCER MOD %s" % (text("title_team")))
	popup.addline(" ")
	popup.addline("%s:" % (es.ServerVar("sm_match_name_t")))
	popup.addline(" ")

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.ServerVar("sm_stats_pos_t"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popup.addline("%s: %s" % (text("possession"), percentage))

	values = ["goals", "assists", "own_goals", "hits", "passes", "interceptions"]

	cvars = ["sm_stats_number_goals_t", "sm_stats_number_assists_t", "sm_stats_number_own_goals_t", "sm_stats_number_hits_t", "sm_stats_number_passes_t", "sm_stats_number_interceptions_t"]
	for i in range(0, len(values)):
		popup.addline("%s: %s" % (text(values[i]), es.ServerVar(cvars[i])))

	popup.addline(" ")
	popup.addline("%s:" % (es.ServerVar("sm_match_name_ct")))
	popup.addline(" ")

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.ServerVar("sm_stats_pos_ct"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popup.addline("%s: %s" % (text("possession"), percentage))

	cvars = ["sm_stats_number_goals_ct", "sm_stats_number_assists_ct", "sm_stats_number_own_goals_ct", "sm_stats_number_hits_ct", "sm_stats_number_passes_ct", "sm_stats_number_interceptions_ct"]
	for i in range(0, len(values)):
		popup.addline("%s: %s" % (text(values[i]), es.ServerVar(cvars[i])))

	popup.addline(" ")
	popup.addline("->9. %s" % (common("back")))
	popup.addline(" ")
	popup.addline("0. %s" % (common("close")))

	for i in range(1, 9):
		popup.select(i, team)

	popup.submenu(9, "sm_popup_stats")

	popuplib.send(name, userid)

def personal(userid, choice, popupid):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.exists("key", "soccer_mod_stats", es.getplayersteamid(userid)) == 1:
		create_key_menu(userid, "title_personal")
	else:
		es.tell(userid, "#green", messages("no_stats", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

def player(userid, choice, popupid):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	myPopup = popuplib.easymenu(name, None, player_choice)
	myPopup.settitle(messages("select_player"))

	for player in playerlib.getPlayerList("#human"):
		myPopup.addoption(player.userid, player.name)

	myPopup.submenu(10, "sm_popup_stats")

	popuplib.send(name, userid)

def score_sheet(userid, choice, popupid):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.ServerVar("sm_stats_score_sheet_number") > 1:
		popuplib.send("sm_popup_stats_score_sheet_1", userid)
	else:
		es.tell(userid, "#green", messages("empty", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

# ******************** Easymenu choice ********************
def player_choice(userid, choice, keymenuname):
	tokens = {}
	player = playerlib.getPlayer(userid)

	if es.exists("key", "soccer_mod_stats", es.getplayersteamid(choice)) == 1:
		create_key_menu(userid, "title_player", choice)
	else:
		name = "sm_popup_%s" % (userid)
		if popuplib.exists(name):
			popuplib.delete(name)

		es.tell(userid, "#green", messages("no_stats", tokens, player.get("lang")))
		popuplib.send("sm_popup_stats", userid)

# ******************** Functions ********************
def lang_msg(text, tokens):
	for player in playerlib.getPlayerList("#human"):
		es.tell(player.userid, "#green", messages(text, tokens, player.get("lang")))

def ball_hit():
	if es.ServerVar("sm_stats_round_end") == 0 and es.ServerVar("match_paused") == 0 and es.ServerVar("period_break_running") == 0:
		es.set("sm_match_first_hit", 0)

		userid = int(es.getcmduserid())
		steamid = es.getplayersteamid(userid)
		team = int(es.getplayerteam(userid))
		name = es.getplayername(userid)

		if es.exists("key", "soccer_mod_stats", steamid) == 0:
			es.keycreate("soccer_mod_stats", steamid)

			values = ["possession", "goals", "assists", "own_goals", "hits", "passes", "interceptions"]
			for item in values:
				es.keysetvalue("soccer_mod_stats", steamid, item, 0)

		add_stat(steamid, team, "hits", es.ServerVar("sm_match_rank_hit"))

		shooter_userid = int(es.ServerVar("sm_stats_shooter_userid"))
		shooter_steamid = es.ServerVar("sm_stats_shooter_steamid")
		shooter_team = int(es.ServerVar("sm_stats_shooter_team"))
		shooter_name = es.ServerVar("sm_stats_shooter_name")

		if userid != shooter_userid:
			es.set("sm_stats_assister_userid", shooter_userid)
			es.set("sm_stats_assister_steamid", shooter_steamid)
			es.set("sm_stats_assister_team", shooter_team)
			es.set("sm_stats_assister_name", shooter_name)

			es.set("sm_stats_shooter_userid", userid)
			es.set("sm_stats_shooter_steamid", steamid)
			es.set("sm_stats_shooter_team", team)
			es.set("sm_stats_shooter_name", name)

			global shooter_time_stamp
			global assister_time_stamp
			assister_time_stamp = shooter_time_stamp
			shooter_time_stamp = time.time()

			if shooter_userid > 0:
				if team == shooter_team:
					add_stat(shooter_steamid, shooter_team, "passes", es.ServerVar("sm_match_rank_pass"))
				else:
					add_stat(steamid, team, "interceptions", es.ServerVar("sm_match_rank_interception"))

				possession = int(shooter_time_stamp - assister_time_stamp)
				if possession > 0:
					add_possession(shooter_steamid, shooter_team, possession)

def show_stats(winner):
	es.set("sm_match_first_hit", 1)
	es.set("sm_stats_round_end", 1)
	es.set("sm_stats_winner", winner)

	shooter_userid = int(es.ServerVar("sm_stats_shooter_userid"))
	shooter_team = int(es.ServerVar("sm_stats_shooter_team"))
	shooter_steamid = es.ServerVar("sm_stats_shooter_steamid")
	shooter_name = es.ServerVar("sm_stats_shooter_name")

	assister_userid = int(es.ServerVar("sm_stats_assister_userid"))
	assister_team = int(es.ServerVar("sm_stats_assister_team"))
	assister_steamid = es.ServerVar("sm_stats_assister_steamid")
	assister_name = es.ServerVar("sm_stats_assister_name")

	if int(es.ServerVar("sm_match_goal_scored")) == 1:
		if shooter_userid > 0:
			current = time.time()
			global shooter_time_stamp
			possession = int(current - shooter_time_stamp)

			if possession > 0:
				add_possession(shooter_steamid, shooter_team, possession)

			tokens = {}
			if shooter_team == winner:
				tokens["name"] = shooter_name
				lang_msg("goal", tokens)

				add_stat(shooter_steamid, shooter_team, "goals", es.ServerVar("sm_match_rank_goal"))

				if assister_team == winner:
					tokens["name"] = assister_name
					lang_msg("assist", tokens)

					add_stat(assister_steamid, assister_team, "assists", es.ServerVar("sm_match_rank_assist"))
			else:
				tokens["name"] = shooter_name
				lang_msg("own_goal", tokens)

				add_stat(shooter_steamid, shooter_team, "own_goals", es.ServerVar("sm_match_rank_own_goal"))

			es.doblock("soccer_mod_2010/match/stats/score_sheet/add")

	if es.ServerVar("sm_match_rank_save_round_end") == 1:
		connection.commit()

	load_variables()

def add_stat(steamid, team, type, points):
	if team == 2:
		name = "sm_stats_number_%s_t" % (type)
	elif team == 3:
		name = "sm_stats_number_%s_ct" % (type)
	es.set(name, es.ServerVar(name) + 1)

	es.keysetvalue("soccer_mod_stats", steamid, type, int(es.keygetvalue("soccer_mod_stats", steamid, type)) + 1)

	if es.ServerVar("sm_match_started") == 1:
		points *= int(es.ServerVar("sm_match_rank_multiplier"))

	points *= es.getplayercount(2) + es.getplayercount(3)

	cursor.execute("UPDATE players SET %s = (%s + 1), rank = (rank + %s) WHERE steamid = '%s'" % (type, type, points, steamid))

def add_possession(steamid, team, possession):
	if team == 2:
		es.set("sm_stats_pos_t", es.ServerVar("sm_stats_pos_t") + possession)
	elif team == 3:
		es.set("sm_stats_pos_ct", es.ServerVar("sm_stats_pos_ct") + possession)

	es.set("sm_stats_total_pos", es.ServerVar("sm_stats_total_pos") + possession)

	es.keysetvalue("soccer_mod_stats", steamid, "possession", int(es.keygetvalue("soccer_mod_stats", steamid, "possession")) + possession)

def create_key_menu(userid, title, choice = 1):
	name = "sm_popup_%s" % (userid)
	if popuplib.exists(name):
		popuplib.delete(name)

	popup = popuplib.create(name)

	popup.addline("SOCCER MOD %s" % (text(title)))
	popup.addline(" ")

	if choice > 1:
		player_name = es.getplayername(choice)
		popup.addline("%s:" % (player_name))
		popup.addline(" ")

		steamid = es.getplayersteamid(choice)
	else:
		steamid = es.getplayersteamid(userid)

	possession = float(es.ServerVar("sm_stats_total_pos"))
	if possession > 0:
		value = float(es.keygetvalue("soccer_mod_stats", steamid, "possession"))
		percentage = str(int(round(((value / possession) * 100), 0))) + "%"
	else:
		percentage = "0%"

	popup.addline("%s: %s" % (text("possession"), percentage))

	stats = ["goals", "assists", "own_goals", "hits", "passes", "interceptions"]
	count = 0
	for stat in stats:
		value = float(es.keygetvalue("soccer_mod_stats", steamid, stats[count]))
		value_t = float(es.ServerVar("sm_stats_number_%s_t" % (stats[count])))
		value_ct = float(es.ServerVar("sm_stats_number_%s_ct" % (stats[count])))

		total = value_t + value_ct
		if total > 0:
			percentage = value / total
			percentage *= 100
			number = round(percentage, 0)
			percentage = str(int(number)) + "%"
		else:
			percentage = "0%"

		value = int(value)
		total = int(total)

		popup.addline("%s: %s / %s (%s)" % (text(stat), value, total, percentage))
		count += 1
		
	popup.addline(" ")
	popup.addline("->9. %s" % (common("back")))
	popup.addline(" ")
	popup.addline("0. %s" % (common("close")))

	for i in range(1, 9):
		popup.submenu(i, name)

	popup.submenu(9, "sm_popup_stats")

	popuplib.send(name, userid)

def reset_team_stats():
	stats = [
		"sm_stats_pos_t",
		"sm_stats_pos_ct",
		"sm_stats_total_pos",
		"sm_stats_number_goals_t",
		"sm_stats_number_goals_ct",
		"sm_stats_number_assists_t",
		"sm_stats_number_assists_ct",
		"sm_stats_number_own_goals_t",
		"sm_stats_number_own_goals_ct",
		"sm_stats_number_hits_t",
		"sm_stats_number_hits_ct",
		"sm_stats_number_passes_t",
		"sm_stats_number_passes_ct",
		"sm_stats_number_interceptions_t",
		"sm_stats_number_interceptions_ct"
	]

	for item in stats:
		es.set(item, 0)

def set_connected(name, steamid):
	current = datetime.datetime.now()

	if current.month < 10:
		month = "0%s" % (current.month)
	else:
		month = current.month

	if current.day < 10:
		day = "0%s" % (current.day)
	else:
		day = current.day

	if current.hour < 10:
		hour = "0%s" % (current.hour)
	else:
		hour = current.hour

	if current.minute < 10:
		minute = "0%s" % (current.minute)
	else:
		minute = current.minute

	if current.second < 10:
		second = "0%s" % (current.second)
	else:
		second = current.second

	date = "%s-%s-%s %s:%s:%s" % (current.year, month, day, hour, minute, second)

	cursor.execute("SELECT steamid FROM players WHERE steamid = '%s'" % (steamid))

	check = 0
	for row in cursor:
		check += 1

	if check < 1:
		cursor.execute("INSERT INTO players (name, steamid, connected) VALUES ('Encoding error', '%s', '%s')" % (steamid, int(time.time())))

	cursor.execute("UPDATE players SET name = ?, connected = ? WHERE steamid = ?", (name, int(time.time()), steamid))

def execute_query(query, value = None):
	cursor.execute(query)

	if value != None:
		return cursor

# ******************** Unload ********************
def unload():
	connection.commit()
	connection.close()

	es.keygroupdelete("soccer_mod_stats")

	popuplib.delete("sm_popup_stats")

	es.unregclientcmd(es.ServerVar("sm_event_stats"))

	es.unload("soccer_mod_2010/match/stats/ranking")
	es.unload("soccer_mod_2010/match/stats/score_sheet")